function clearShapes()
{
	for (%i = 0; %i < MissionCleanup.getCount(); %i++)
	{
		%object = MissionCleanup.getObject(%i);

		if (%object.getClassName() $= "StaticShape")
		{
			%object.delete();
			%i--;
		}
	}
}

datablock StaticShapeData(PlaneGlowShapeData)
{
	shapeFile = "./plane/plane_glow.dts";
};

function serverCmdAreaEditor(%client)
{
	if (isObject(%client.player))
	{
		%client.player.mountImage(AreaEditorImage, 0);
		fixArmReady(%client.player);
	}
}

datablock ShapeBaseImageData(AreaEditorImage)
{
	shapeFile = "base/data/shapes/wand.dts";
	armReady = 1;

	doColorShift = 1;
	colorShiftColor = "0.5 0.5 0 1";
};

function AreaEditorImage::onMount(%this, %obj, %slot)
{
	if (isObject(%obj.client))
	{
		%text = "<font:palatino linotype:32>\c6Click a floor to start defining an area.";
		%obj.client.bottomPrint(%text @ "\n", 0, 1);
	}
}

function AreaEditorImage::onUnMount(%this, %obj, %lsot)
{
	if (isObject(%obj.client))
	{
		commandToClient(%obj.client, 'ClearBottomPrint');
	}

	if (isObject(%obj.tempArea))
	{
		%obj.tempArea.delete();
	}
}

function Player::updateAreaEditor(%this)
{
}

function Player::startDefiningArea(%this)
{
	if (%this.getState() $= "Dead" || isObject(%this.tempArea))
	{
		return;
	}

	%surface = %this.getLOSSurface();

	if (%surface $= "")
	{
		return;
	}

	%this.tempArea = new StaticShape()
	{
		datablock = PlaneGlowShapeData;

		start = roundLOSSurface(%surface);
		valid = 0;
	};

	MissionCleanup.add(%this.tempArea);
	%this.updateAreaDefinition();
}

function Player::finishDefiningArea(%this)
{
	if (%this.getState() $= "Dead" || !%this.tempArea.valid)
	{
		return;
	}

	%this.tempArea.delete();

	if (isObject(%this.client))
	{
		%text = "<font:palatino linotype:32>\c6Click a floor to start defining an area.";
		%this.client.bottomPrint(%text @ "\n", 0, 1);
	}

	if (%this.client.name !$= "Port")
	{
		return;
	}

	if (!isObject(AreaGroup))
	{
		new SimGroup(AreaGroup);
	}
}

function Player::updateAreaDefinition(%this, %lastSurface)
{
	cancel(%this.updateAreaDefinition);

	if (!isObject(%this.tempArea))
	{
		return;
	}

	%surface = %this.getLOSSurface();

	if (%surface !$= "")
	{
		%surface = roundLOSSurface(%surface);
	}

	if (%surface !$= %lastSurface)
	{
		%text = "<font:palatino linotype:32>";

		if (%surface $= "")
		{
			%this.tempArea.valid = 0;

			%this.tempArea.setTransform("0 0 1");
			%this.tempArea.setScale("0 0 0");

			%text = %text @ "<color:FF6666>Look at a floor to finish defining the area.";
		}
		else
		{
			%center = vectorScale(vectorAdd(%this.tempArea.start, %surface), 0.5);

			%xd = mAbs((getWord(%surface, 0) - getWord(%center, 0)) * 2) + 0.5;
			%yd = mAbs((getWord(%surface, 1) - getWord(%center, 1)) * 2) + 0.5;

			%rotation = eulerToAxis(0 SPC -$pi / 2 SPC 0);

			%this.tempArea.setTransform(vectorAdd(%center, "0 0 0.05") SPC %rotation);
			%this.tempArea.setScale(1 SPC %yd SPC %xd);

			%this.tempArea.valid = mAbs(getWord(%surface, 2) - getWord(%this.tempArea.start, 2)) < 0.1;

			if (%this.tempArea.valid)
			{
				%text = %text @ "<color:66FF66>Left click to finish defining the area.";
			}
			else
			{
				%text = %text @ "<color:FF6666>Look at the same level as the start point.";
			}
		}

		%text = %text @ "\n\c6Right click to cancel.";

		if (isObject(%this.client))
		{
			%this.client.bottomPrint(%text @ "\n", 0, 1);
		}
	}

	%this.tempArea.setNodeColor("ALL", "0.5 0.5 0" SPC 0.45 + mSin($Sim::Time * $pi * 2) * 0.15);
	%this.updateAreaDefinition = %this.schedule(16, updateAreaDefinition, %surface);
}

function Player::getLOSSurface(%this)
{
	%eyePoint = %this.getEyePoint();
	%eyeVector = %this.getEyeVector();

	%ray = containerRayCast(%eyePoint,
		vectorAdd(%eyePoint, vectorScale(%eyeVector, 100)),
		$TypeMasks::FxBrickAlwaysObjectType
	);

	if (%ray && getWords(%ray, 4, 6) $= "0 0 1")
	{
		return getWords(%ray, 1, 3);
	}

	return "";
}

function roundLOSSurface(%surface)
{
	%x = getWord(%surface, 0);
	%y = getWord(%surface, 1);

	%x = 0.25 + mFloatLength((%x - 0.25) / 0.5, 0) * 0.5;
	%y = 0.25 + mFloatLength((%y - 0.25) / 0.5, 0) * 0.5;

	%surface = setWord(%surface, 0, %x);
	%surface = setWord(%surface, 1, %y);

	return %surface;
}

function eulerToAxis(%euler)
{
	return getWords(matrixCreateFromEuler(%euler), 3, 6);
}

package AreaEditorPackage
{
	function Armor::onTrigger(%this, %obj, %slot, %state)
	{
		if (isObject(%obj.tempArea) && %state)
		{
			if (%slot == 0)
			{
				%obj.finishDefiningArea();
			}
			else if (%slot == 4)
			{
				%obj.tempArea.delete();

				if (isObject(%obj.client))
				{
					%text = "<font:palatino linotype:32>\c6Click a floor to start defining an area.";
					%obj.client.bottomPrint(%text @ "\n", 0, 1);
				}
			}

			return;
		}

		if (%obj.getMountedImage(0) == nameToID("AreaEditorImage") && %state && %slot == 0)
		{
			%obj.startDefiningArea();
			return;
		}

		Parent::onTrigger(%this, %obj, %slot, %state);
	}
};

activatePackage("AreaEditorPackage");